import unittest

import libthai_cffi

TEST_STR = "สวัสดีครับ กอ.รมน. นี่เป็นการทดสอบตัวเอง"


class BreakTestCase(unittest.TestCase):
    def test_find_breaks(self):
        self.assertEqual(libthai_cffi.find_breaks(TEST_STR), [6, 11, 19, 22, 26, 29, 34])

    def test_find_breaks_text(self):
        self.assertEqual(
            libthai_cffi.find_breaks_text(TEST_STR),
            ["สวัสดี", "ครับ ", "กอ.รมน. ", "นี่", "เป็น", "การ", "ทดสอบ", "ตัวเอง"],
        )


class BreakerTestCase(unittest.TestCase):
    def setUp(self):
        self.breaker = libthai_cffi.Breaker()

    def test_find_breaks(self):
        self.assertEqual(self.breaker.find_breaks(TEST_STR), [6, 11, 19, 22, 26, 29, 34])

    def test_find_breaks_text(self):
        self.assertEqual(
            self.breaker.find_breaks_text(TEST_STR),
            ["สวัสดี", "ครับ ", "กอ.รมน. ", "นี่", "เป็น", "การ", "ทดสอบ", "ตัวเอง"],
        )



import cffi

ffibuilder = cffi.FFI()
ffibuilder.cdef(
    """
typedef wchar_t thwchar_t;
typedef ... ThBrk;

ThBrk *th_brk_new(const char *dictpath);
void th_brk_delete(ThBrk *brk);
int th_brk_wc_find_breaks (ThBrk *brk, const thwchar_t *s, int pos[], size_t pos_sz);
size_t th_wnormalize(thwchar_t dest[], const thwchar_t *src, size_t n);
"""
)
ffibuilder.set_source(
    "libthai_cffi._libthai",
    """
#include <thai/thwchar.h>
#include <thai/thwbrk.h>
#include <thai/thwstr.h>
""",
    libraries=["thai"],
)



from setuptools import find_packages, setup

setup(
    name="libthai-cffi",
    version="1.0.1",
    url="https://github.com/whs/libthai-cffi",
    author="Manatsawin Hanmongkolchai",
    author_email="manatsawin+pypi@gmail.com",
    description="CFFI Binding to libthai",
    license="LGPL",
    packages=find_packages(exclude=["test"]),
    setup_requires=["cffi~=1.11.5"],
    install_requires=["cffi~=1.11.5"],
    cffi_modules=["package/build_cffi.py:ffibuilder"],
)



from . import _libthai, exception


class Breaker:
    def __init__(self, dictpath=_libthai.ffi.NULL):
        breaker = _libthai.lib.th_brk_new(dictpath)

        if breaker == _libthai.ffi.NULL:
            raise exception.InitError

        self._breaker = _libthai.ffi.gc(breaker, _libthai.lib.th_brk_delete)

    def find_breaks(self, text, count=100):
        positions = _libthai.ffi.new("int[]", count)

        actual_count = _libthai.lib.th_brk_wc_find_breaks(self._breaker, text, positions, count)

        out = []
        for i in range(actual_count):
            out.append(positions[i])

        return out

    def find_breaks_text(self, text, count=100):
        breaks = self.find_breaks(text, count)
        out = []

        last_index = 0

        for index in breaks:
            out.append(text[last_index:index])
            last_index = index

        out.append(text[last_index:])

        return out


def find_breaks(text, count=100):
    return Breaker().find_breaks(text, count)


def find_breaks_text(text, count=100):
    return Breaker().find_breaks_text(text, count)


def normalize(text):
    size = len(text)
    out = _libthai.ffi.new("thwchar_t[]", size)
    _libthai.lib.th_wnormalize(out, text, size)

    return _libthai.ffi.string(out, size)
